/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.upgrade;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.olat.basesecurity.BaseSecurity;
import org.olat.core.commons.persistence.DB;
import org.olat.core.id.Roles;
import org.olat.core.util.StringHelper;
import org.olat.core.util.nodes.INode;
import org.olat.course.CourseFactory;
import org.olat.course.ICourse;
import org.olat.course.Structure;
import org.olat.course.condition.Condition;
import org.olat.course.nodes.AbstractAccessableCourseNode;
import org.olat.course.nodes.AbstractFeedCourseNode;
import org.olat.course.nodes.BCCourseNode;
import org.olat.course.nodes.CalCourseNode;
import org.olat.course.nodes.CourseNode;
import org.olat.course.nodes.DialogCourseNode;
import org.olat.course.nodes.FOCourseNode;
import org.olat.course.nodes.GenericCourseNode;
import org.olat.course.nodes.InfoCourseNode;
import org.olat.course.nodes.PortfolioCourseNode;
import org.olat.course.nodes.ProjectBrokerCourseNode;
import org.olat.course.nodes.STCourseNode;
import org.olat.course.nodes.TACourseNode;
import org.olat.course.nodes.WikiCourseNode;
import org.olat.course.run.scoring.ScoreCalculator;
import org.olat.course.tree.CourseEditorTreeModel;
import org.olat.course.tree.CourseEditorTreeNode;
import org.olat.group.BusinessGroup;
import org.olat.group.BusinessGroupImpl;
import org.olat.group.BusinessGroupService;
import org.olat.group.area.BGArea;
import org.olat.group.area.BGAreaImpl;
import org.olat.group.area.BGAreaManager;
import org.olat.group.area.BGtoAreaRelationImpl;
import org.olat.group.context.BGContext2Resource;
import org.olat.group.model.BGResourceRelation;
import org.olat.repository.RepositoryEntry;
import org.olat.repository.RepositoryManager;
import org.olat.repository.SearchRepositoryEntryParameters;
import org.olat.resource.OLATResource;
import org.olat.resource.OLATResourceManager;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * Description:<br>
 * upgrade code for OLAT 7.1.0 -> OLAT 7.1.1
 * - fixing invalid structures being built by synchronisation, see OLAT-6316 and OLAT-6306
 * - merges all yet found data to last valid node 
 * 
 * <P>
 * Initial Date: 24.03.2011 <br>
 * 
 * @author Roman Haag, roman.haag@frentix.com, www.frentix.com
 */
public class OLATUpgrade_8_2_0 extends OLATUpgrade {

	private static final String TASK_CONTEXTS = "Upgrade contexts";
	private static final String TASK_AREAS = "Upgrade areas";
	private static final String TASK_CONDITIONS = "Upgrade conditions";
	private static final int REPO_ENTRIES_BATCH_SIZE = 20;
	private static final String VERSION = "OLAT_8.2.0";
	
	@Autowired
	private DB dbInstance;
	@Autowired
	private BaseSecurity securityManager;
	@Autowired
	private BGAreaManager areaManager;
	@Autowired
	private BusinessGroupService businessGroupService;
	@Autowired
	private OLATResourceManager resourceManager;
	@Autowired
	private RepositoryManager repositoryManager;

	public OLATUpgrade_8_2_0() {
		super();
	}

	@Override
	public String getVersion() {
		return VERSION;
	}

	@Override
	public boolean doPreSystemInitUpgrade(UpgradeManager upgradeManager) {
		return false;
	}

	@Override
	public boolean doPostSystemInitUpgrade(UpgradeManager upgradeManager) {
		UpgradeHistoryData uhd = upgradeManager.getUpgradesHistory(VERSION);
		if (uhd == null) {
			// has never been called, initialize
			uhd = new UpgradeHistoryData();
		} else {
			if (uhd.isInstallationComplete()) {
				return false;
			}
		}
		
		boolean allOk = upgradeGroups(upgradeManager, uhd);
		allOk &= upgradeAreas(upgradeManager, uhd);
		allOk &= upgradeCourseConditions(upgradeManager, uhd);

		uhd.setInstallationComplete(allOk);
		upgradeManager.setUpgradesHistory(uhd, VERSION);
		if(allOk) {
			log.audit("Finished OLATUpgrade_8_2_0 successfully!");
		} else {
			log.audit("OLATUpgrade_8_2_0 not finished, try to restart OpenOLAT!");
		}
		return allOk;
	}
	
	private boolean upgradeGroups(UpgradeManager upgradeManager, UpgradeHistoryData uhd) {
		if (!uhd.getBooleanDataValue(TASK_CONTEXTS)) {
			int counter = 0;
			List<BusinessGroup> groups;
			do {
				groups = findBusinessGroups(counter, REPO_ENTRIES_BATCH_SIZE);
				for(BusinessGroup group:groups) {
					processBusinessGroup(group);
				}
				counter += groups.size();
				log.audit("Processed context: " + groups.size());
			} while(groups.size() == REPO_ENTRIES_BATCH_SIZE);
			
			uhd.setBooleanDataValue(TASK_CONTEXTS, true);
			upgradeManager.setUpgradesHistory(uhd, VERSION);
		}
		return true;
	}
	
	private boolean upgradeAreas(UpgradeManager upgradeManager, UpgradeHistoryData uhd) {
		if (!uhd.getBooleanDataValue(TASK_AREAS)) {
			int counter = 0;
			List<BGAreaImpl> areas;
			do {
				areas = findAreas(counter, REPO_ENTRIES_BATCH_SIZE);
				for(BGAreaImpl area:areas) {
					processArea(area);
				}
				counter += areas.size();
				log.audit("Processed areas: " + areas.size());
			} while(areas.size() == REPO_ENTRIES_BATCH_SIZE);
			
			uhd.setBooleanDataValue(TASK_AREAS, true);
			upgradeManager.setUpgradesHistory(uhd, VERSION);
		}
		return true;
	}
	
	private boolean upgradeCourseConditions(UpgradeManager upgradeManager, UpgradeHistoryData uhd) {
		if (!uhd.getBooleanDataValue(TASK_CONDITIONS)) {
			int counter = 0;
			List<RepositoryEntry> entries;
			SearchRepositoryEntryParameters params = new SearchRepositoryEntryParameters();
			params.setRoles(new Roles(true, false, false, false, false, false, false));
			params.addResourceTypes("CourseModule");
			do {
				entries = repositoryManager.genericANDQueryWithRolesRestriction(params, counter, REPO_ENTRIES_BATCH_SIZE, true);
				for(RepositoryEntry entry:entries) {
					processCourse(entry);
				}
				counter += entries.size();
				log.audit("Processed repository entries: " + entries.size());
			} while(entries.size() == REPO_ENTRIES_BATCH_SIZE);
			
			uhd.setBooleanDataValue(TASK_CONDITIONS, false);
			upgradeManager.setUpgradesHistory(uhd, VERSION);
		}
		return true;
	}
	
	private void processCourse(RepositoryEntry entry) {
		ICourse opendedCourse = null;
		try {
			List<BusinessGroup> groups = businessGroupService.findBusinessGroups(null, entry.getOlatResource(), 0, -1);
			List<BGArea> areas = areaManager.findBGAreasInContext(entry.getOlatResource());

			ICourse course = CourseFactory.loadCourse(entry.getOlatResource());
			ProcessingOccured processingFlag = new ProcessingOccured();
			log.info("Start: " + course.getCourseTitle());
			Structure structure = course.getRunStructure();
			processCourseRec(processingFlag, structure.getRootNode(), groups, areas);
			CourseEditorTreeModel editorTree = course.getEditorTreeModel();
			processEditorCourseRec(processingFlag, editorTree.getRootNode(), groups, areas);
			if(processingFlag.isProcessed()) {
				opendedCourse = CourseFactory.openCourseEditSession(course.getResourceableId());
				log.info("Save of:" + course.getCourseTitle() + " (" + course.getResourceableId() + ")");
				CourseFactory.saveCourse(opendedCourse.getResourceableId());
			} else {
				log.info("No change for: " + course.getCourseTitle() + " (" + course.getResourceableId() + ")");
			}
		} catch (Exception e) {
			log.error("", e);
		} finally {
			if(opendedCourse != null) {
				CourseFactory.closeCourseEditSession(opendedCourse.getResourceableId(), true);
			}	
		}
	}
	
	private void processCourseRec(ProcessingOccured processingFlag, INode node,
			List<BusinessGroup> groups, List<BGArea> areas) {
		if(node instanceof CourseNode) {
			processCourseNode(processingFlag, (CourseNode)node, groups, areas);
		}
		for(int i=node.getChildCount(); i-->0; ) {
			INode subNode = node.getChildAt(i);
			processCourseRec(processingFlag, subNode, groups, areas);
		}
	}
	
	private void processEditorCourseRec(ProcessingOccured processingFlag, INode node,
			List<BusinessGroup> groups, List<BGArea> areas) {
		if(node instanceof CourseEditorTreeNode) {
			processCourseNode(processingFlag, ((CourseEditorTreeNode)node).getCourseNode(), groups, areas);
		}
		for(int i=node.getChildCount(); i-->0; ) {
			INode subNode = node.getChildAt(i);
			processEditorCourseRec(processingFlag, subNode, groups, areas);
		}
	}
	
	/**
	 * Update all the conditions in every type of course node
	 * @param processingFlag
	 * @param node
	 * @param groups
	 * @param areas
	 */
	private void processCourseNode(ProcessingOccured processingFlag, CourseNode node,
			List<BusinessGroup> groups, List<BGArea> areas) {
		if(node instanceof GenericCourseNode) {
			GenericCourseNode genericNode = (GenericCourseNode)node;
			processCondition(processingFlag, genericNode.getPreConditionAccess(), groups, areas);
			processCondition(processingFlag, genericNode.getPreConditionVisibility(), groups, areas);
			if(node instanceof AbstractAccessableCourseNode) {
				//nothing self but
				
				if(node instanceof WikiCourseNode) {
					WikiCourseNode wikiNode = (WikiCourseNode)node;
					processCondition(processingFlag, wikiNode.getPreConditionEdit(), groups, areas);
				} else if(node instanceof STCourseNode) {
					STCourseNode structureNode = (STCourseNode)node;
					ScoreCalculator calculator = structureNode.getScoreCalculator();
					String score = calculator.getScoreExpression();
					String passed = calculator.getPassedExpression();
					
					boolean stProcessed = false;
					if(StringHelper.containsNonWhitespace(score)) {
						String processedExpression = processExpression(processingFlag, score, groups, areas);
						if(!processedExpression.equals(score)) {
							calculator.setScoreExpression(processedExpression);
							stProcessed = true;
						}	
					}
					
					if(StringHelper.containsNonWhitespace(passed)) {
						String processedExpression = processExpression(processingFlag, passed, groups, areas);
						if(!processedExpression.equals(score)) {
							calculator.setScoreExpression(processedExpression);
							stProcessed = true;
						}	
					}
					
					if(stProcessed) {
						structureNode.setScoreCalculator(calculator);
					}
				} else if (node instanceof PortfolioCourseNode) {
					PortfolioCourseNode portfolioNode = (PortfolioCourseNode)node;
					processCondition(processingFlag, portfolioNode.getPreConditionEdit(), groups, areas);
				} else if (node instanceof InfoCourseNode) {
					InfoCourseNode infoNode = (InfoCourseNode)node;
					processCondition(processingFlag, infoNode.getPreConditionEdit(), groups, areas);
					processCondition(processingFlag, infoNode.getPreConditionAdmin(), groups, areas);
				} else if (node instanceof DialogCourseNode) {
					DialogCourseNode dialogNode = (DialogCourseNode)node;
					processCondition(processingFlag, dialogNode.getPreConditionModerator(), groups, areas);
					processCondition(processingFlag, dialogNode.getPreConditionPoster(), groups, areas);
					processCondition(processingFlag, dialogNode.getPreConditionReader(), groups, areas);
				} else if (node instanceof CalCourseNode) {
					CalCourseNode calNode = (CalCourseNode)node;
					processCondition(processingFlag, calNode.getPreConditionEdit(), groups, areas);
				}	
			} 
			if(node instanceof TACourseNode) {
				TACourseNode taskNode = (TACourseNode)node;
				processCondition(processingFlag, taskNode.getConditionDrop(), groups, areas);
				processCondition(processingFlag, taskNode.getConditionReturnbox(), groups, areas);
				processCondition(processingFlag, taskNode.getConditionScoring(), groups, areas);
				processCondition(processingFlag, taskNode.getConditionSolution(), groups, areas);
				processCondition(processingFlag, taskNode.getConditionTask(), groups, areas);
			} else if(node instanceof ProjectBrokerCourseNode) {
				ProjectBrokerCourseNode brokerNode = (ProjectBrokerCourseNode)node;
				processCondition(processingFlag, brokerNode.getConditionDrop(), groups, areas);
				processCondition(processingFlag, brokerNode.getConditionReturnbox(), groups, areas);
				processCondition(processingFlag, brokerNode.getConditionScoring(), groups, areas);
				processCondition(processingFlag, brokerNode.getConditionProjectBroker(), groups, areas);
			} else if (node instanceof FOCourseNode) {
				FOCourseNode forumNode = (FOCourseNode)node;
				processCondition(processingFlag, forumNode.getPreConditionModerator(), groups, areas);
				processCondition(processingFlag, forumNode.getPreConditionPoster(), groups, areas);
				processCondition(processingFlag, forumNode.getPreConditionReader(), groups, areas);
			} else if (node instanceof BCCourseNode) {
				BCCourseNode bcNode = (BCCourseNode)node;
				processCondition(processingFlag, bcNode.getPreConditionDownloaders(), groups, areas);
				processCondition(processingFlag, bcNode.getPreConditionUploaders(), groups, areas);
			} else if (node instanceof AbstractFeedCourseNode) {
				AbstractFeedCourseNode feedNode = (AbstractFeedCourseNode)node;
				processCondition(processingFlag, feedNode.getPreConditionModerator(), groups, areas);
				processCondition(processingFlag, feedNode.getPreConditionPoster(), groups, areas);
				processCondition(processingFlag, feedNode.getPreConditionReader(), groups, areas);
			}
		}
	}
	
	private void processCondition(ProcessingOccured processingFlag, Condition condition,
			List<BusinessGroup> groups, List<BGArea> areas) {
		boolean easy = StringHelper.containsNonWhitespace(condition.getConditionFromEasyModeConfiguration());
		if(easy) {
			//already processed?
			if(StringHelper.containsNonWhitespace(condition.getEasyModeGroupAccessIds())
					|| StringHelper.containsNonWhitespace(condition.getEasyModeGroupAreaAccessIds())) {
				return;
			}
			if(!StringHelper.containsNonWhitespace(condition.getEasyModeGroupAccess())
					&& !StringHelper.containsNonWhitespace(condition.getEasyModeGroupAreaAccess())) {
				return;
			}
			
			String groupKeys = toGroupKeys(condition.getEasyModeGroupAccess(), groups);
			condition.setEasyModeGroupAccessIds(groupKeys);
			String areaKeys = toAreaKeys(condition.getEasyModeGroupAreaAccess(), areas);
			condition.setEasyModeGroupAreaAccessIds(areaKeys);
			processingFlag.processed();
		} else if(condition.isExpertMode()) {
			String expression = condition.getConditionExpression();
			if(StringHelper.containsNonWhitespace(expression)) {
				String reference = condition.getConditionExpression();
				String processExpression = processExpression(processingFlag, expression, groups, areas);
				if(!reference.equals(processExpression)) {
					condition.setConditionUpgraded(processExpression);
					processingFlag.processed();
				}
			}
		}
	}
	
	private String processExpression(ProcessingOccured processingFlag, String expression,
			List<BusinessGroup> groups, List<BGArea> areas) {

		for(BusinessGroup group:groups) {
			String strToMatch = "\"" + group.getName() + "\"";
			String replacement = "\"" + group.getKey() + "\"";
			expression = replaceAllCaseInsensitive(expression, strToMatch, replacement);
		}
		for(BGArea area:areas) {
			String strToMatch = "\"" + area.getName() + "\"";
			String replacement = "\"" + area.getKey() + "\"";
			expression = replaceAllCaseInsensitive(expression, strToMatch, replacement);
		}
		
		return expression;
	}
	
	private String replaceAllCaseInsensitive(String expression, String name, String replacement) {
		String lcExpresion = expression.toLowerCase();
		String lcName = name.toLowerCase();

		int index = 0;
		while((index = lcExpresion.indexOf(lcName, index)) > 0) {
			int startIndex = index;
			int stopIndex = index + lcName.length();
			
			String newExpression = expression.substring(0, startIndex);
			newExpression += replacement;
			newExpression += expression.substring(stopIndex);
			
			expression = newExpression;
			lcExpresion = expression.toLowerCase();
			index = startIndex + replacement.length();	
		}
		return expression;
	}
	
	private String toGroupKeys(String groupNames, List<BusinessGroup> groups) {
		if(!StringHelper.containsNonWhitespace(groupNames)) return null;
		String[] groupNameArr = groupNames.split(",");
		StringBuilder sb = new StringBuilder();
		for(String groupName:groupNameArr) {
			groupName = groupName.trim();
			for(BusinessGroup group:groups) {
				if(groupName.equalsIgnoreCase(group.getName())) {
					if(sb.length() > 0) {
						sb.append(',');
					}
					sb.append(group.getKey());
					break;
				}
			}
		}
		return sb.toString().trim();
	}
	
	private String toAreaKeys(String areaNames, List<BGArea> areas) {
		if(!StringHelper.containsNonWhitespace(areaNames)) return null;
		String[] areaNameArr = areaNames.split(",");
		StringBuilder sb = new StringBuilder();
		for(String areaName:areaNameArr) {
			areaName = areaName.trim();
			for(BGArea area:areas) {
				if(areaName.equalsIgnoreCase(area.getName())) {
					if(sb.length() > 0) {
						sb.append(',');
					}
					sb.append(area.getKey());
					break;
				}
			}
		}
		return sb.toString().trim();
	}
	
	private void processBusinessGroup(BusinessGroup group) {
		List<OLATResource> resources = findOLATResourcesForBusinessGroup(group);
		List<OLATResource> currentList = businessGroupService.findResources(Collections.singletonList(group), 0, -1);
		
		boolean merge = false;
		if(group.getResource() == null) {
			OLATResource resource = resourceManager.findResourceable(group);
			((BusinessGroupImpl)group).setResource(resource);
			merge = true;
		}
		if(group.getOwnerGroup() == null) {
			((BusinessGroupImpl)group).setOwnerGroup(securityManager.createAndPersistSecurityGroup());
			merge = true;
		}
		if(group.getPartipiciantGroup() == null) {
			((BusinessGroupImpl)group).setPartipiciantGroup(securityManager.createAndPersistSecurityGroup());
			merge = true;
		}
		if(group.getWaitingGroup() == null) {
			group.setWaitingGroup(securityManager.createAndPersistSecurityGroup());
			merge = true;
		}
		if(merge) {
			group = dbInstance.getCurrentEntityManager().merge(group);
		}

		int count = 0;
		for	(OLATResource resource:resources) {
			if(!currentList.contains(resource)) {
				businessGroupService.addResourceTo(group, resource);
				count++;
			}
		}
		dbInstance.commitAndCloseSession();
		log.audit("Processed: " + group.getName() + " add " + count + " resources");
	}
	
	private void processArea(BGAreaImpl area) {
		if(area.getResource() != null) {
			//already migrated
			return;
		}
		Long groupContextKey = area.getGroupContextKey();
		List<OLATResource> resources = findOLATResourcesForBGContext(groupContextKey);
		if(resources.isEmpty()) {
			//nothing to do
		} else {
			//reuse the area for the first resource
			Iterator<OLATResource> resourcesIt = resources.iterator();
			OLATResource firstResource = resourcesIt.next();
			area.setResource(firstResource);
			dbInstance.getCurrentEntityManager().merge(area);
			List<Long> firstResourcesGroupKeys = findBusinessGroupsOfResource(firstResource);
			List<BusinessGroup> originalGroupList = findBusinessGroupsOfArea(area);
			//remove the groups which aren't part of the first resource
			for(BusinessGroup group:originalGroupList) {
				if(!firstResourcesGroupKeys.contains(group.getKey())) {
					areaManager.removeBGFromArea(group, area);
				}
			}
			
			//duplicate the areas for the next resources
			if(resourcesIt.hasNext()) {
				for( ;resourcesIt.hasNext(); ) {
					OLATResource resource = resourcesIt.next();
					List<Long> resourcesGroupKeys = findBusinessGroupsOfResource(resource);
					BGArea existingArea = areaManager.findBGArea(area.getName(), resource);
					if(existingArea == null) {
						BGArea copyArea = areaManager.createAndPersistBGAreaIfNotExists(area.getName(), area.getDescription(), resource);
						for(BusinessGroup group:originalGroupList) {
							if(resourcesGroupKeys.contains(group.getKey())) {
								areaManager.addBGToBGArea(group, copyArea);
							}
						}
					}
				}
			}
		}
		dbInstance.commitAndCloseSession();
	}
	
	private List<OLATResource> findOLATResourcesForBGContext(Long contextKey) {
		StringBuilder q = new StringBuilder();
		q.append("select bgcr.resource from ").append(BGContext2Resource.class.getName()).append(" as bgcr where bgcr.groupContext.key=:contextKey");
		
		List<OLATResource> resources = dbInstance.getCurrentEntityManager()
				.createQuery(q.toString(), OLATResource.class)
				.setParameter("contextKey", contextKey)
				.getResultList();
		return resources;
	}
	
	private List<Long> findBusinessGroupsOfResource(OLATResource resource) {
		StringBuilder sb = new StringBuilder();
		sb.append("select distinct relation.group.key from ").append(BGResourceRelation.class.getName()).append(" relation where relation.resource.key=:resourceKey");

		List<Long> groups = dbInstance.getCurrentEntityManager()
				.createQuery(sb.toString(), Long.class)
				.setParameter("resourceKey", resource.getKey())
				.getResultList();
		return groups;
	}
	
	private List<BusinessGroup> findBusinessGroupsOfArea(BGArea area) {
		StringBuilder q = new StringBuilder();
		q.append("select bgarel.businessGroup from ").append(BGtoAreaRelationImpl.class.getName()).append(" as bgarel ")
		 .append(" where bgarel.groupArea.key=:areaKey");
		
		List<BusinessGroup> groups = dbInstance.getCurrentEntityManager()
				.createQuery(q.toString(), BusinessGroup.class)
				.setParameter("areaKey", area.getKey())
				.getResultList();
		return groups;
	}
	
	private List<BGAreaImpl> findAreas(int firstResult, int maxResults) {
		StringBuilder q = new StringBuilder();
		q.append("select area from ").append(BGAreaImpl.class.getName()).append(" area ")
		 .append(" left join fetch area.resource resource")
		 .append(" order by area.key");

		List<BGAreaImpl> resources = dbInstance.getCurrentEntityManager().createQuery(q.toString(), BGAreaImpl.class)
				.setFirstResult(firstResult)
				.setMaxResults(maxResults)
				.getResultList();
		return resources;
	}
	
	private List<BusinessGroup> findBusinessGroups(int firstResult, int maxResults) {
		StringBuilder q = new StringBuilder();
		q.append("select bg from ").append(BusinessGroupImpl.class.getName()).append(" bg ")
		 .append(" left join fetch bg.ownerGroup onwerGroup")
		 .append(" left join fetch bg.partipiciantGroup participantGroup")
		 .append(" left join fetch bg.waitingGroup waitingGroup")
		 .append(" order by bg.key");

		List<BusinessGroup> resources = dbInstance.getCurrentEntityManager().createQuery(q.toString(), BusinessGroup.class)
				.setFirstResult(firstResult)
				.setMaxResults(maxResults)
				.getResultList();
		return resources;
	}
	
	private List<OLATResource> findOLATResourcesForBusinessGroup(BusinessGroup group) {
		StringBuilder q = new StringBuilder();
		q.append("select bgcr.resource from ").append(BGContext2Resource.class.getName()).append(" as bgcr where bgcr.groupContext.key=:contextKey");

		List<OLATResource> resources = dbInstance.getCurrentEntityManager().createQuery(q.toString(), OLATResource.class)
				.setParameter("contextKey", ((BusinessGroupImpl)group).getGroupContextKey())
				.getResultList();
		return resources;
	}
	
	private class ProcessingOccured {
		private boolean processed = false;

		public boolean isProcessed() {
			return processed;
		}

		public void processed() {
			processed = true;
		}
	}
}
